use std::{
    fmt,
    ops::{Add, AddAssign, Mul},
};

#[derive(Clone, Copy)]
#[repr(C)]
pub struct Rgba {
    r: f32,
    g: f32,
    b: f32,
    a: f32,
}

pub const fn rgba(r: f32, g: f32, b: f32, a: f32) -> Rgba {
    Rgba { r, g, b, a }
}

impl Rgba {
    pub const BLACK: Self = rgba(0.0, 0.0, 0.0, 1.0);
}

impl fmt::Debug for Rgba {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        f.debug_tuple(stringify!(Rgba))
            .field(&self.r)
            .field(&self.g)
            .field(&self.b)
            .field(&self.a)
            .finish()
    }
}

impl Add for Rgba {
    type Output = Self;

    fn add(self, Self { r, g, b, a }: Self) -> Self::Output {
        Self {
            r: self.r + r,
            g: self.g + g,
            b: self.b + b,
            a: self.a + a,
        }
    }
}

impl AddAssign for Rgba {
    fn add_assign(&mut self, Self { r, g, b, a }: Self) {
        self.r += r;
        self.g += g;
        self.b += b;
        self.a += a;
    }
}

impl Mul<f32> for Rgba {
    type Output = Self;

    fn mul(self, rhs: f32) -> Self::Output {
        Self {
            r: self.r * rhs,
            g: self.g * rhs,
            b: self.b * rhs,
            a: self.a * rhs,
        }
    }
}

impl Mul<Rgba> for f32 {
    type Output = Rgba;

    fn mul(self, rhs: Rgba) -> Self::Output {
        rhs * self
    }
}
